# gradox/ARAST.R
# pipeline in R similar to MG-RAST 20180310
# (up to RNA and protein genecalling steps)

# usage: to process a batch of files:
# files <- dir("/home/ARAST/work/", ".fasta", full.names=TRUE)
# system.time(lapply(files, process))
# stop after the dereplication step and don't gzip the output:
# system.time(lapply(files, process, run="DNA", run.gzip=FALSE))

# 'run' argument specified which steps to run:
# DNA: trimming, denoising, artifact removal
# RNA: DNA steps plus RNA genecalling (sortmerna)
# all: DNA and RNA steps plus protein genecalling (FragGeneScan)
# FGS: FragGeneScan only

# SOFTWARE: taken from MG-RAST's "Processing Information and Downloads" for ERX242607_sra_data.fasta
# download skewer_0.2.2 from https://github.com/relipmoc/skewer
# download bowtie_v2.3.4.1 from https://github.com/BenLangmead/bowtie2 [used by autoskewer]
# download autoskewer (ZIP) from https://github.com/MG-RAST/autoskewer
# download ea-utils (ZIP) from https://github.com/ExpressionAnalysis/ea-utils [fastq-mcf, used by filter_sequences]
# install biopython [used by seq_length_stats.py]
#   sbopkg: upgrade pip, then run pip install biopython
# download pipeline (ZIP) from https://github.com/MG-RAST/pipeline
#   [seq_length_stats.py, filter_sequences, dereplication.py, parallel_FragGeneScan.py]
# download sortmerna (ZIP) from https://github.com/biocore/sortmerna
# download FragGeneScan_FGS1.18 from https://github.com/MG-RAST/FGS

# DATABASE:
# download md5rna.clust (pipeline-master/CWL/Inputs/DBs/getpredata.sh) [used by sortmerna]
#   curl "http://shock.metagenomics.anl.gov/node/c4c76c22-297b-4404-af5c-8cd98e580f2a?download" > md5rna.clust
# set up index for sortmerna:
#   indexdb --ref md5rna.clust,md5rna.clust.index

# 20180315 differences between processing files:
# office computer gives smaller rna files than menehune
# Baltic_Sea/ERX2426*
# StLawrence_Estuary/ERX985695 (only one)
# Peru_Margin/SRX000334, SRX000335

techtypes <- list(
  `454` = c("ERX242602", "ERX242602", "ERX242603", "ERX242603", "ERX242604", "ERX242604", "ERX242605", "ERX242605",
            "ERX242606", "ERX242606", "ERX242607", "ERX242607", "SRX025906", "SRX025907", "SRX025908", "SRX025909",
            "SRX025910", "SRX025911", "SRX025912", "SRX025913", "SRX080938", "SRX080950", "SRX080951", "SRX080962",
            "SRX095384", "SRX095389", "SRX095391", "SRX095392", "SRX1008212", "SRX139859", "SRX139866", "SRX143928",
            "SRX146990", "SRX024780", "SRX024838", "SRX024839", "SRX024841", "SRX024842", "SRX000331", "SRX000332", 
            "SRX000333", "SRX000334", "SRX000335", "SRX002155", "SRX000174", "SRX002157", "SRX002159", "SRX002156",
            "SRX000175", "SRX002158", "SRX002160", "SRX202013", "SRX202014", "SRX021587", "SRX021589", "SRX021591",
            "SRX021587", "SRX345845", "SRX345844", "SRX345843", "SRX339932", "SRX339931", "SRX339930", "SRX339929",
            "SRX339928", "SRX025760", "SRX025796", "SRX025825", "SRX331735", "SRX012322",
            "SRX024781", "SRX024782", "SRX024783", "SRX024784", "SRX024843"),
  illumina = c("SRX648440", "SRX648442", "SRX648443", "SRX648444", "SRX648445", "SRX854074", "SRX854078", "SRX854081",
               "SRX854083", "SRX854084", "SRX895490", "SRX895491", "SRX895493", "SRX895494", "SRX895496", "SRX134768",
               "SRX134769", "SRX511174", "SRX511256", "SRX511268", "SRX511269", "SRX511271", "SRX511273", "SRX511275",
               "SRX511277", "SRX511303", "SRX511304", "SRX511305", "SRX511306", "SRX1012820", "SRX1012825", "SRX1012836",
               "SRX1012840", "SRX1012844", "SRX958530", "SRX958772", "SRX958773", "SRX1527166", "SRX1527804", "SRX1527805", 
               "SRX1527806", "SRX1527807", "SRX1527808", "SRX1527809", "SRX1527810", "SRX1527811", "SRX1527812",
               "SRX707230", "SRX706859", "SRX706431", "SRX706003", "SRX704264",
               "SRX348621", "SRX373050", "SRX373052", "SRX373056", "SRX373060", "SRX373062",
               "SRX373068", "SRX373070", "SRX373074", "SRX373064", "SRX373080",
               "SRX187180", "SRX330851", "SRX331883", "SRX331884", "SRX331956", "SRX331958",
               "SRX648310", "SRX648290", "SRX648311", "SRX648296", "SRX648313", "SRX643298",
               "SRX3188062", "SRX2869655", "SRX2869654", "SRX2869653",
               "SRX1660383", "SRX1660385", "SRX1660387", "SRX1660389",
               "ERX1158314", "ERX1158315", "ERX1158316",
               "SRX268151", "SRX268150", "SRX268149",
               "ERX1749068", "ERX1749067", "ERX1749066", "ERX1749065", "ERX1749064",
               "SRX2334639", "SRX2334707", "SRX2334640", "SRX2334668", "SRX2334697", "SRX2334652", "SRX2334669", "SRX2334641"),
  torrent = c("SRX1337936", "SRX1337939", "ERX985693", "ERX985694", "ERX985695", "ERX985696"),
  solid = c("SRX021586", "SRX021588", "SRX021590", "SRX021592", "SRX233537", "SRX233569", "SRX233675", "SRX233692",
            "SRX233711", "SRX233721", "SRX233727", "SRX233741", "SRX233751", "SRX233770", "SRX233781", "SRX233797",
            "SRX233805", "SRX233812", "SRX233829", "SRX233839", "SRX233846", "SRX233855", "SRX233862", "SRX233866", 
            "SRX233877", "SRX233888", "SRX233898")
)

# function to process a single file 20180310
process <- function(file, proc=parallel::detectCores(), rna_nr="/home/ARAST/REFDB/md5rna.clust", run="all", run.gzip=TRUE) {
  ## what steps to run
  if(run=="all") steps <- c(0:9, 23:24)
  if(run=="DNA") steps <- c(0:4, 23:24)
  if(run=="RNA") steps <- c(0:6, 23:24)
  if(run=="FGS") {
    steps <- c(0, 9, 23:24)
    no_rRNAfile <- file
  }
  if(run=="afterDNA") {
    steps <- c(0, 5:9, 23:24)
    derepfile <- file
  }
  if(0 %in% steps) {
    print("######## 0. Set up environment ########")
    # get workdir and ID from file name
    workdir <- dirname(file)
    ID <- strsplit(basename(file), "_")[[1]][1]
    # file type: fasta or fastq
    if(grepl("\\.fasta$", file)) filetype <- "fasta"
    else if(grepl("\\.fna$", file)) filetype <- "fasta"
    else if(grepl("\\.fastq$", file)) filetype <- "fastq"
    else {
      print(paste("not processing", file, "which is not fasta or fastq"))
      return()
    }
    # type of sequencing technology (needed for FragGeneScan)
    techtype <- names(techtypes)[sapply(techtypes, function(x) any(grepl(ID, x)))]
    # empty techtype, Ion Torrent, or SOLiD is treated as 454
    if(length(techtype)==0) techtype <- "454"
    else if(techtype == "torrent") techtype <- "454"
    else if(techtype == "solid") techtype <- "illumina"
    print(paste0("processing ", filetype, " file for ", ID, " (", techtype, " sequencing)"))
    tmpdir <- file.path(workdir, "tmp")
    if(!dir.exists(tmpdir)) dir.create(tmpdir)
    # keep list of temporary (to remove) and output (to gzip) files
    tmpfiles <- outfiles <- character()
  }
  # NOTE: 00000000 steps are omitted from/do not follow MG-RAST pipeline
  if(1 %in% steps) print("00000000 1. Initial sequence statistics 00000000")
  if(2 %in% steps) {
    print("######## 2. Adapter Trimming ########")
    scrubbedfile <- file.path(workdir, paste0(ID, "_scrubbed.", filetype))
    autoskewer(file, scrubbedfile, tmpdir)
    tmpfiles <- c(tmpfiles, scrubbedfile)
  }
  if(3 %in% steps) {
    print("######## 3. Denoising and normalization ########")
    passedfile <- file.path(workdir, paste0(ID, "_passed.fasta"))
    removedfile <- file.path(workdir, paste0(ID, "_removed.fasta"))
    preprocess(scrubbedfile, passedfile, removedfile, filetype)
    tmpfiles <- c(tmpfiles, passedfile, removedfile)
  }
  if(4 %in% steps) {
    print("######## 4. Removal of sequencing artifacts ########")
    derepfile <- file.path(workdir, paste0(ID, "_dereplicated.fasta"))
    dereplication(passedfile, derepfile, tmpdir)
    outfiles <- c(outfiles, derepfile)
  }
  if(5 %in% steps) print("00000000 5. Host DNA contamination removal 00000000")
  if(6 %in% steps) {
    print("######## 6. RNA feature identification (rRNA genecalling) ########")
    index <- paste0(rna_nr, ".index")
    rRNAfile <- file.path(workdir, paste0(ID, "_rRNA.fasta"))
    no_rRNAfile <- file.path(workdir, paste0(ID, "_no-rRNA.fasta"))
    # adjust memory use based on number of processors
    if(proc > 4) mem <- 6144 else mem <- 2048 
    sortmerna(derepfile, rRNAfile, no_rRNAfile, proc, mem, rna_nr, index)
    outfiles <- c(outfiles, rRNAfile, no_rRNAfile)
  }
  if(7 %in% steps) print("00000000 7. RNA clustering 00000000")
  if(8 %in% steps) print("00000000 8. RNA similarity search 00000000")
  if(9 %in% steps) {
    print("######## 9. Identify putative protein coding features (genecalling) ########")
    codingfile <- file.path(workdir, paste0(ID, "_coding"))
    genecalling(no_rRNAfile, codingfile, techtype, tmpdir, proc)
    outfiles <- c(outfiles, paste0(codingfile, ".fna"), paste0(codingfile, ".faa"))
  }
  if(23 %in% steps) {
    print("00000000 23. Summary statistics 00000000")
    # gzip output files and remove intermediate files
    system(paste("rm -f", paste(tmpfiles, collapse=" ")))
    if(run.gzip) system(paste("gzip -f", paste(outfiles, collapse=" ")))
  }
  if(24 %in% steps) print("######## 24. Completed ########")
}

# run autoskewer.py 20180310
autoskewer <- function(file, scrubbedfile, tmpdir) {
  cmd <- paste("autoskewer.py -v -i", file, "-o", scrubbedfile, "-l ~/tmp/scrubbed.log -t", tmpdir)
  system(cmd)
  # clean up tmpdir
  tmpfiles <- file.path(tmpdir, "*")
  cmd <- paste("rm", tmpfiles)
  system(cmd)
}

# run filter_sequences 20180309
# based on MG-RAST/pipeline/mgcmd/mgrast_preprocess.pl
preprocess <- function(file, passedfile, removedfile, filetype) {
  # print stats
  cmd <- paste("seq_length_stats.py -i", file, "-t", filetype, "-f")
  system(cmd)
  if(filetype=="fasta") {
    # get stats
    cmd <- paste("seq_length_stats.py -i", file, "-t", filetype, "-f | cut -f2")
    out <- system(cmd, intern=TRUE)
    # this gives bp_count, sequence_count, average_length, standard_deviation_length, length_min, length_max
    out <- as.numeric(out)
    # translated from mgrast_preprocess.pl
    mymean <- out[3]
    mystdv <- out[4]
    min_ln <- round(mymean - 2 * mystdv)
    max_ln <- round(mymean + 2 * mystdv)
    if(min_ln < 0) min_ln <- 0
    cmd <- paste("filter_sequences -i", file, "-format fasta -o", passedfile, "-r", removedfile,
                  "-filter_ln -min_ln", min_ln, "-max_ln", max_ln, "-filter_ambig -max_ambig 5")
  }
  if(filetype=="fastq") {
    cmd <- paste("filter_sequences -i", file, "-format fastq -o", passedfile, "-r", removedfile,
                  "-dynamic_trim -min_qual 15 -max_lqb 5")
  }
  system(cmd)
}

# run dereplication.py 20180310
# based on MG-RAST/pipeline/mgcmd/mgrast_dereplicate.pl
dereplication <- function(file, derepfile, tmpdir) {
  prefix_size <- 50
  memory <- 8  # GB
  run_dir <- tmpdir
  input <- file
  out_prefix <- tmpdir
  cmd <- paste("dereplication.py -l", prefix_size, "-m", memory, "-d", run_dir, input, out_prefix)
  system(cmd)
  # rename and clean up files
  system(paste("mv", paste0(tmpdir, ".passed.fna"), derepfile))
  system(paste("rm", paste0(tmpdir, ".*")))
}

# run sortmerna 20180310
# based on MG-RAST/pipeline/mgcmd/mgrast_sortme_rna.pl
sortmerna <- function(file, rRNAfile, no_rRNAfile, proc, mem, rna_nr, index) {
  eval <- 0.1
  ident <- 70 # cutoff value listed in the MG-RAST Processing Information page
  fasta <- file
  ## NOTE: MG-RAST picks sequences using the BLAST ouput with an identity threshold;
  ##   here we use sortmerna with command modified for FASTA output (--fastx),
  ##   fast filtering (--num_alignments), and saving both aligned and rejected sequences;
  ##   sortmerna doesn't have an identity threshold option
  #ref <- paste0(rna_nr, ",", index)
  #cmd <- paste("sortmerna -a", proc, "-m", mem, "-e", eval,
  #             "--fastx --num_alignments 1 --ref", ref, "--reads", fasta,
  #             "--aligned", rRNAfile, "--other", no_rRNAfile, "-v")
  # 20180311: use a modified version of mgrast_sortme_rna.pl to save both RNA and non-RNA sequences
  cmd <- paste("./arast_sortme_rna.pl -input", file, "-output", rRNAfile, "-notrna", no_rRNAfile,
               "-rna_nr", rna_nr, "-index", index, "-proc", proc, "-mem", mem, "-eval", eval, "-ident", ident)
  system(cmd)
  # clean up files
  WD <- setwd(dirname(file))
  system("rm *.blast *.tab *.id *.sort")
  setwd(WD)
}

# run FragGeneScan 20180309
# based on MG-RAST/pipeline/mgcmd/mgrast_genecalling.pl
genecalling <- function(file, codingfile, techtype=c("sanger", "454", "illumina"), tmpdir, proc) {
  out_prefix <- codingfile
  # sequencing technology: mgrast_genecalling.pl chooses training files with highest error rate
  if(techtype=="sanger") type <- "sanger_10"
  if(techtype=="454") type <- "454_30"
  if(techtype=="illumina") type <- "illumina_10"
  cmd <- paste("parallel_FragGeneScan.py -v -p", proc, "-s 100 -t", type, "-d", tmpdir, file, out_prefix) 
  # sometimes(?) that gives python error:
  # OSError: [Errno 2] No such file or directory
  #cmd <- paste("run_FragGeneScan.pl -genome", file, "-out", codingfile, "-complete 0 -train", type) 
  system(cmd)
  # clean up files
  system(paste("rm", paste0(codingfile, ".out")))
  system(paste("rm", file.path(tmpdir, "*")))
}
