# gradox/kraken.R
# run Kraken and extract sequences for abundant species  20180527

# usage:
# run_kraken("Diffuse_Vents", "SRA_MG")              # produce *.kraken and *.report files in /home/gradox/Diffuse_Vents/Kraken/
# find_species("Diffuse_Vents", "SRA_MG")            # list abundant species in *.report files
# extract_reads("Diffuse_Vents", "SRA_MG", 1427364)  # extract reads for the specified taxid to a new FASTA file
## for getting reads from multiple taxids (e.g. Candidatus Thioglobus singularis and the strain PS1):
# extract_reads("Diffuse_Vents", "SRA_MG", c(1427364, 1125411))  

# source MG.R for mprep() 
source("MG.R")

# location of Kraken database
DBNAME <- "/home/Kraken/minikraken_20171101_8GB_dustmasked"

run_kraken <- function(study, seqtype, preload=TRUE) {
  # get names of input FASTA files and samples
  mp <- mprep(study, seqtype, doit=FALSE)
  # make Kraken directory
  DIR <- file.path(mp$basedir, "Kraken")
  system(paste("mkdir -p", DIR))
  # loop over input FASTA files
  for(i in 1:length(mp$infiles)) {
    outfile <- paste0(DIR, "/", mp$IDs[i], ".kraken")
    # uncompressed FASTA files produced by ARAST (to dereplication step) are in bigARAST directory
    fastafile <- gsub("sra_data", "dereplicated", gsub(".gz$", "", gsub("/SRA", "/bigARAST", mp$infiles[i])))
    # command to run Kraken
    if(preload) cmd <- paste("kraken --preload --threads 8 --db", DBNAME, fastafile, ">", outfile)
    else cmd <- paste("kraken --threads 8 --db", DBNAME, fastafile, ">", outfile)
    print(cmd)
    system(cmd)
    ## command to filter Kraken output
    #filtfile <- paste0(DIR, "/", mp$IDs[i], ".filter")
    #cmd <- paste("kraken-filter --db", DBNAME, "--threshold 0.01", outfile, ">", filtfile)
    #print(cmd)
    #system(cmd)
    # command to make Kraken report
    repfile <- paste0(DIR, "/", mp$IDs[i], ".report")
    cmd <- paste("kraken-report --db", DBNAME, outfile, ">", repfile)
    print(cmd)
    system(cmd)
  }
}

# find species that are relatively abundant in all samples
find_species <- function(study, seqtype, percent=1) {
  mp <- mprep(study, seqtype, doit=FALSE)
  DIR <- file.path(mp$basedir, "Kraken")
  repfiles <- paste0(file.path(mp$basedir, "Kraken", mp$IDs), ".report")
  for(i in 1:length(repfiles)) {
    dat <- read.csv(repfiles[i], as.is=TRUE, sep="\t", header=FALSE, col.names=c("percentage", "nreads_clade", "nreads_taxon", "rank", "taxid", "name"))
    # extract rows for species or "-" (subspecies)
    ispecies <- dat$rank=="S"
    # exclude "root" and "cellular organisms"
    isub <- dat$rank=="-" & dat$taxid != 1 & dat$taxid != 131567
    dat <- dat[ispecies | isub, ]
    # the sum of reads classified to any taxon
    nreads <- sum(dat$nreads_taxon)
    # the percentage for each taxon
    frac <- dat$nreads_taxon / nreads
    # replace the 'percentage' column (all reads) with the calculated values (classified reads)
    dat$percentage <- round(100*frac, 2)
    # extract rows with greater than specified percent representation
    dat <- dat[frac > percent/100, ]
    # print the sample name and identified species
    print(mp$samples[i])
    print(dat)
    # assemble the output for writing to a file
    thisout <- cbind(sample = mp$samples[i], dat)
    if(i==1) out <- thisout else out <- rbind(out, thisout)
  }
  file <- paste0("one_percent/", study, "_", seqtype, ".csv")
  write.csv(out, file, row.names=FALSE)
  invisible(out)
}

# extract reads classified as the indicated species (taxid)  20180528
extract_reads <- function(study, seqtype, taxid) {
  mp <- mprep(study, seqtype, doit=FALSE)
  DIR <- file.path(mp$basedir, "Kraken")
  krakenfiles <- paste0(file.path(mp$basedir, "Kraken", mp$IDs), ".kraken.gz")
  for(i in 1:length(krakenfiles)) {
    # for faster re-running of this function, look for stored kraken data
    kraken <- gsub(".gz$", "", basename(krakenfiles[i]))
    if(exists(kraken)) {
      print(paste("--- using previously read classifications from", kraken, "---"))
      dat <- get(kraken)
    } else {
      print(paste("--- reading classifications from", krakenfiles[i], "---"))
      dat <- read.csv(krakenfiles[i], as.is=TRUE, sep="\t", header=FALSE, col.names=c("status", "seqID", "taxid", "length", "LCA_mapping"))
      # store the data for later use
      assign(kraken, dat, envir = .GlobalEnv)
    }
    # extract rows for specified taxid
    dat <- dat[dat$taxid %in% taxid, ]
    if(nrow(dat)==0) print("------ SKIPPED - NO READS!") else {
      # write seqIDs
      write.table(dat$seqID, "/tmp/seqID.txt", row.names=FALSE, col.names=FALSE, quote=FALSE)
      # untruncated and uncompressed FASTA files are in bigSRA directory
      #fastafile <- gsub(".gz$", "", gsub("/SRA", "/bigSRA", mp$infiles[i]))
      fastafile <- gsub("sra_data", "dereplicated", gsub(".gz$", "", gsub("/SRA", "/bigARAST", mp$infiles[i])))
      # extract the sequences to a file with the first taxid appended to the SRA ID
      fastaname <- gsub(".gz$", "", basename(mp$infiles[i]))
      outfile <- file.path(DIR, gsub(mp$IDs[i], paste0(mp$IDs[i], "-", taxid[1]), fastaname))
      # perl trick from http://bioinformatics.cvr.ac.uk/blog/short-command-lines-for-manipulation-fastq-and-fasta-sequence-files/
      # escape backslash to workaround Error: '\S' is an unrecognized escape in character string starting ""perl -ne 'if(/^>(\S"
      #cmd <- paste("perl -ne 'if(/^>(\S+)/){$c=$i{$1}}$c?print:chomp;$i{$_}=1 if @ARGV' /tmp/seqID.txt", fastafile, ">", outfile)
      cmd <- paste("perl -ne 'if(/^>(\\S+)/){$c=$i{$1}}$c?print:chomp;$i{$_}=1 if @ARGV' /tmp/seqID.txt", fastafile, ">", outfile)
      print(cmd)
      system(cmd)
    }
  }
}
